﻿using Azure;
using FluentNHibernate;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using System.Diagnostics;
using TravelCompanyBusinessLogic.BusinessLogic.TouristLogics;
using TravelCompanyBusinessLogic.BusinessLogics;
using TravelCompanyClientApp.Models;
using TravelCompanyContracts.BindingModels;
using TravelCompanyContracts.BindingModels.OperatorBindingModels;
using TravelCompanyContracts.BindingModels.TouristBindingModels;
using TravelCompanyContracts.BusinessLogicsContracts;
using TravelCompanyContracts.BusinessLogicsContracts.OperatorBusinessLogicsContracts;
using TravelCompanyContracts.BusinessLogicsContracts.TouristBusinessLogicsContracts;
using TravelCompanyContracts.SearchModels.OperatorSearchModels;
using TravelCompanyContracts.SearchModels.TouristSearchModels;
using TravelCompanyContracts.ViewModels;
using TravelCompanyContracts.ViewModels.OperatorViewModels;
using TravelCompanyContracts.ViewModels.TouristViewModels;
using TravelCompanyDatabaseImplement.Models.TouristModel;
using TravelCompanyDataModels.Models.OperatorModels;
using VisioForge.MediaFramework.Helpers;

namespace TravelCompanyClientApp.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;
        private readonly ITouristLogic _logic;
        private readonly ITravelLogic _travellogic;
        private readonly IReportLogic _reportlogic;
        private readonly IOperatorLogic _operatorlogic;
        private readonly ITourLogic _tourLogic;
        private readonly ITouristLogic _touristLogic;
        private readonly IGuideLogic _guidelogic;

        public HomeController(ILogger<HomeController> logger, ITouristLogic logic, ITravelLogic travelLogic, IReportLogic reportLogic
            , IOperatorLogic operatorLogic, ITourLogic tourLogic, IGuideLogic guideLogic,
            ITouristLogic touristLogic)
        {
            _logger = logger;
            _logic = logic;
            _travellogic = travelLogic;
            _reportlogic = reportLogic;
            _operatorlogic = operatorLogic;
            _tourLogic = tourLogic;
            _guidelogic = guideLogic;
            LoadFromJson();
            _touristLogic = touristLogic;
        }

        [HttpGet]
        public IActionResult Privacy()
        {
            if (APIClient.Tourist == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.Tourist);
        }
        [HttpPost]
        public void Privacy(string login, string email, string password, string surname, string name, string patronymic, string birthday)
        {
            if (APIClient.Tourist == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(email)
                || string.IsNullOrEmpty(surname) || string.IsNullOrEmpty(name) || string.IsNullOrEmpty(patronymic) || string.IsNullOrEmpty(birthday))
            {
                throw new Exception("Неверно введены личные данные");
            }
            _logic.Update(new TouristBindingModel
            {
                Id = APIClient.Tourist.Id,
                Surname = surname,
                Name = name,
                Patronymic = patronymic,
                BirthDay = DateTime.Parse(birthday),
                Login = login,
                Email = login,
                Password = password
            });
            APIClient.Tourist.Surname = surname;
            APIClient.Tourist.Name = name;
            APIClient.Tourist.Patronymic = patronymic;
            APIClient.Tourist.BirthDay = DateTime.Parse(birthday);
            APIClient.Tourist.Login = login;
            APIClient.Tourist.Email = email;
            APIClient.Tourist.Password = password;
            Response.Redirect("Privacy");
        }

        [HttpGet]
        public IActionResult Enter()
        {

            return View();
        }

        [HttpPost]
        public void Enter(string login, string password)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password))
            {
                throw new Exception("Введите логин и пароль");
            }
            APIClient.Tourist = _logic.ReadElement(new TouristSearchModel
            {
                Login = login,
                Password = password
            });
            if (APIClient.Tourist == null)
            {
                throw new Exception("Неверный логин/пароль");
            }
            Response.Redirect("Privacy");
        }

        [HttpGet]
        public IActionResult Report()
        {
            if (APIClient.Tourist == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.TravelList = _travellogic.ReadList(new TravelSearchModel { TouristId = APIClient.Tourist.Id });
            return View(new ReportBindingModel());
        }

        [HttpPost]
        public IActionResult Report(ReportBindingModel model)
        {
            model.TouristId = APIClient.Tourist.Id;
            List<ReportGuidesPCView> travels = _reportlogic.GetGuidesPCView(new ReportBindingModel()
            {
                DateAfter = model.DateAfter,
                DateBefore = model.DateBefore,
                TouristId = model.TouristId
            });
            ViewBag.TravelList = _travellogic.ReadList(new TravelSearchModel { TouristId = APIClient.Tourist.Id });
            ViewBag.Travels = travels;
            return View(new ReportBindingModel());
        }


        [HttpPost]
        public void ReportPdf(ReportBindingModel model)
        {
            model.TouristId = APIClient.Tourist.Id;
            _reportlogic.saveGuidesToPdfFile(new ReportBindingModel()
            {
                DateAfter = model.DateAfter,
                DateBefore = model.DateBefore,
                TouristId = model.TouristId
            });
            Response.Redirect("Report");
        }

        [HttpPost]
        public void ReportXsl(ReportBindingModel model)
        {
            model.TouristId = APIClient.Tourist.Id;
            _reportlogic.saveGuidesToExcel(new ReportBindingModel()
            {
                DateAfter = model.DateAfter,
                DateBefore = model.DateBefore,
                TouristId = model.TouristId,
                TravelId = model.TravelId,

            });
            Response.Redirect("Report");
        }

        [HttpPost]
        public void ReportWord(ReportBindingModel model)
        {
            model.TouristId = APIClient.Tourist.Id;
            _reportlogic.saveGuidesToWord(new ReportBindingModel()
            {
                DateAfter = model.DateAfter,
                DateBefore = model.DateBefore,
                TouristId = model.TouristId,
                TravelId = model.TravelId
            });
            Response.Redirect("Report");
        }
        [HttpPost]
        public IActionResult ReportPdfView([FromBody] ReportBindingModel model)
        {
            if (APIClient.Tourist == null)
            {
                return Redirect("~/Home/Enter");
            }

            model.TouristId = APIClient.Tourist.Id;
            List<ReportTravelsPCView> travels = _reportlogic.GetTravelsPCView(new ReportBindingModel()
            {
                DateAfter = model.DateAfter,
                DateBefore = model.DateBefore,
                TouristId = model.TouristId
            });
            ViewBag.TravelList = _travellogic.ReadList(new TravelSearchModel { TouristId = APIClient.Tourist.Id });
            ViewBag.Travels = travels;
            return View(new ReportBindingModel());
        }
        [HttpPost]
           public void ReportEmail(ReportBindingModel model)
           {
               model.TouristId = APIClient.Tourist.Id;
            string path = $"C:\\Users\\Public\\Documents\\{DateTime.Now.ToString("HH.mm.ss_dd.MM.yyyy")}.pdf";
            var sendermodel= new MailSendInfoBindingModel
            {
                MailAddress = APIClient.Tourist.Email,
                Subject = "Отчет",
                report = model,
                Path = path
            };
            _reportlogic.saveGuidesToPdfFile(new ReportBindingModel
            {
                FileName = path,
                TouristId = sendermodel.report.TouristId,
                DateBefore = sendermodel.report.DateBefore,
                DateAfter = sendermodel.report.DateAfter
            });
            _touristLogic.SendMail(sendermodel);
               Response.Redirect("Report");
           }
        [HttpGet]
        public IActionResult Register()
        {
            return View();
        }

        [HttpPost]
        public void Register(string login, string email, string password, string surname, string name, string patronymic, string birthday)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(email)
                 || string.IsNullOrEmpty(surname) || string.IsNullOrEmpty(name) || string.IsNullOrEmpty(patronymic) || string.IsNullOrEmpty(birthday))
            {
                throw new Exception("Неверно введены личные данные");
            }
            _logic.Create(new TouristBindingModel
            {
                Login = login,
                Email = email,
                Password = password,
                Surname = surname,
                Name = name,
                Patronymic = patronymic,
                BirthDay = DateTime.Parse(birthday)
            });
            Response.Redirect("Enter");
            return;
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }

        public List<TourBindingModel> LoadToursFromJson(string jsonFilePath)
        {
            string jsonData = System.IO.File.ReadAllText(jsonFilePath);
            var jsonObject = JsonConvert.DeserializeObject<Dictionary<string, List<TourBindingModel>>>(jsonData);
            var tours = jsonObject["tour"];
            return tours;
        }

        public List<GuideBindingModel> LoadGuidesFromJson(string jsonFilePath)
        {
            string jsonData = System.IO.File.ReadAllText(jsonFilePath);
            var jsonObject = JsonConvert.DeserializeObject<Dictionary<string, List<GuideBindingModel>>>(jsonData);
            var guides = jsonObject["guide"];
            return guides;
        }
        public List<OperatorBindingModel> LoadOperatorsFromJson(string jsonFilePath)
        {
            string jsonData = System.IO.File.ReadAllText(jsonFilePath);
            var jsonObject = JsonConvert.DeserializeObject<Dictionary<string, List<OperatorBindingModel>>>(jsonData);
            var operators = jsonObject["operator"];
            return operators;
        }

        public void LoadFromJson()
        {
            if (_operatorlogic.ReadList(null).Count == 0)
            {
                string jsonFilePath = "automatic.json";
                List<TourBindingModel> tours = LoadToursFromJson(jsonFilePath);
                List<GuideBindingModel> guides = LoadGuidesFromJson(jsonFilePath);
                List<OperatorBindingModel> operators = LoadOperatorsFromJson(jsonFilePath);
                int b = 0;
                foreach (var o in operators)
                {
                    _operatorlogic.Create(o);
                    b = _operatorlogic.ReadElement(new OperatorSearchModel { Login = o.Login }).Id;
                }
                foreach (var t in tours)
                {
                    t.OperatorId = b;
                    _tourLogic.Create(t);
                }

                foreach (var g in guides)
                {
                    g.OperatorId = b;
                    _guidelogic.Create(g);
                }
            }
        }
    }
}